@@ -93,6 +93,7 @@ bool ImportParser::getNextTags(const QString &text, StandardTags &st, int &pos)
 {
 	int idx, oldpos = pos;
 	if (pattern.isEmpty()) {
+		trackDuration.clear();
 		return false;
 	}
 #if QT_VERSION >= 300
@@ -184,7 +185,13 @@ bool ImportParser::getNextTags(const QString &text, StandardTags &st, int &pos)
 		if (genrePos != -1) {
 			QString genreStr = re.cap(genrePos);
 			int genre = Genres::getNumber(genreStr);
-			if (genre != 0xff) st.genre = genre;
+			if (genre != 0xff) {
+				st.genre = genre;
+				st.genreStr = QString::null;
+			} else if (!genreStr.isEmpty()) {
+				st.genre = 0xff;
+				st.genreStr = genreStr;
+			}
 		}
 		pos = idx + re.matchedLength();
 		if (pos > oldpos) { /* avoid endless loop */
@@ -307,7 +314,13 @@ bool ImportParser::getNextTags(const QString &text, StandardTags &st, int &pos)
 		if (idx != -1) {
 			QString genreStr(text.mid(idx + 6, len - 7).stripWhiteSpace());
 			int genre = Genres::getNumber(genreStr);
-			if (genre != 0xff) st.genre = genre;
+			if (genre != 0xff) {
+				st.genre = genre;
+				st.genreStr = QString::null;
+			} else if (!genreStr.isEmpty()) {
+				st.genre = 0xff;
+				st.genreStr = genreStr;
+			}
 			pos = idx + len;
 		}
 	}
