@@ -8,6 +8,7 @@
  */
 
 #include "config.h"
+#include <qfile.h>
 #ifdef CONFIG_USE_KDE
 
 #include <kapp.h>
@@ -41,7 +42,7 @@ int main(int argc, char *argv[])
 	KAboutData aboutData(
 	    "kid3", I18N_NOOP("Kid3"),
 	    VERSION, description, KAboutData::License_GPL,
-	    "(c) 2003-2005 Urs Fleisch", 0, "http://kid3.sourceforge.net",
+	    "(c) 2003-2006 Urs Fleisch", 0, "http://kid3.sourceforge.net",
 	    "ufleisch@users.sourceforge.net");
 	aboutData.addAuthor("Urs Fleisch",0, "ufleisch@users.sourceforge.net");
 	KCmdLineArgs::init(argc, argv, &aboutData);
@@ -59,7 +60,7 @@ int main(int argc, char *argv[])
 			KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
 			if (args->count()) {
-				kid3->openDirectory(args->arg(0));
+				kid3->openDirectory(QFile::decodeName(args->arg(0)));
 			}
 			args->clear();
 		}
@@ -103,7 +104,7 @@ int main(int argc, char *argv[])
 		kid3->show();
 		app.setMainWidget(kid3);
 		if (argc > 1) {
-			kid3->openDirectory(argv[1]);
+			kid3->openDirectory(QFile::decodeName(argv[1]));
 		}
 	}
 	return app.exec();
